import pandas as pd
from dateutil import parser

import quantsc as qsc


def change_plot_backend(backend):
    if backend not in qsc.config.config['supported_plot_backend']:
        raise("Plotting backend must be in " + str(qsc.config.config['supported_plot_backend']))
    qsc.config.config['plot_backend'] = backend

def update_plotly_param():
    if 'get_ipython().__class__.__name__' == 'ZMQInteractiveShell':
        import plotly.offline as pyo
        pyo.init_notebook_mode()

# def round_dates(date_):
def round_dates(dates_array,freq = 'D',parse=True):
    rounded_arr = []
    for time in dates_array:
        if parse:
            time = parser.parse(time)
        time_pd = pd.Timestamp(time).round(freq=freq)
        rounded_arr.append(time_pd)
    return rounded_arr


def seasonal():
    return 1



"""

#https://towardsdatascience.com/time-series-analysis-creating-synthetic-datasets-cf008208e014
def seasonal(range):
    dates = list(pd.data_range(range))
    length = range.length()
    data = np.where(time < 10, time**3, (time-9)**2)
    
    for i in range 5:
        for j in range 50:
            seasonal.append(values[j])

"""


from datetime import datetime, timedelta

# When importing a submodule in the same folder, use this to help
# python know we are importing a submodule instead of a file. Since
# the default behavior looks for __random.py
# import __random as __random
import quantsc.data.__random as __random

def __get_date():
    # np.__random.__random
    pass

def generate_random(range,generate_type = "seasonal"):
    today_str = datetime.today().strftime("%Y-%m-%d")
    start_str = (datetime.today() - timedelta(days=6)).strftime("%Y-%m-%d")
    if generate_type =="seasonal":
        ts = random.seasonal()
        return ts


if __name__ == "__main__":
    print(__random.seasonal())



import numbers

import pandas as pd
import plotly.express as px
# http://theautomatic.net/yahoo_fin-documentation/
# http://theautomatic.net/yahoo_fin-documentation/#methods
import plotly.graph_objects as go
import yahoo_fin.stock_info as si
import yfinance as yf

import quantsc as qsc
import quantsc.config as config
from quantsc.core.timeseries import TimeSeries


class Stock(TimeSeries):
    def __init__(self, ticker=None,start=None,end=None,interval='1d', data=None,name=None):
        if not isinstance(ticker,str) and ticker is not None:
            data = ticker
            ticker = ''
        if data is not None:
            if name is not None:
                self.name = name
                self.ticker = None
            else:
                self.name = "Custom Stock"
            if isinstance(data,TimeSeries):
                self.data = TimeSeries.data
            else:
                super().load_data(data)
                self.dates = self.data.index
        else:
            if interval is not None:
                self.interval = self.is_valid_interval(interval)
            self.ticker = ticker
            self.name = ticker
            try:
                if None in(start,end):
                    stock_data = yf.download(ticker,interval=interval)[:-1]
                else:
                    stock_data = yf.download(ticker,start=start,end=end,interval=interval)[:-1]
            except Exception as e:
                print(str(e))
            series = pd.Series(data=stock_data['Open'],index=stock_data.index)
            super().__init__(series)
            self.open = stock_data['Open']
            self.close = stock_data['Close']
            self.low = stock_data['Low']
            self.high = stock_data['High']
            self.dates = stock_data.index
        self.indicators = dict()
        self.diff_count = 0


    def __add__(self, other):
        if isinstance(other, Stock):
            new_stock = Stock(self.data + other.data,name = f"({self.name}+{other.name})")
            try:
                new_stock.open = self.open + other.open
                new_stock.close = self.close + other.close
                new_stock.high = self.high + other.high
                new_stock.low = self.low + other.low
            finally:
                return new_stock.dropna()
        elif isinstance(other, TimeSeries):
            new_stock = Stock(self.data + other.data,name = f"({self.name}+Series)")
            try:
                new_stock.open = self.open + other.data
                new_stock.close = self.close + other.data
                new_stock.high = self.high + other.data
                new_stock.low = self.low + other.data
            finally:
                return new_stock.dropna()
        elif isinstance(other,numbers.Number):
            new_stock = Stock(self.data + other.data, name = f"{self.name}+{str(other)}")
            try:
                new_stock.high = self.high + other
                new_stock.low = self.low + other
                new_stock.open = self.open + other
                new_stock.close = self.close + other
            finally:
                return new_stock.dropna()

        else:
            raise Exception("Add operation only supported for TimeSeries, Stock, int, and float.")

    def __sub__(self, other):
        if isinstance(other, Stock):
            new_stock = Stock(self.data - other.data, name=f"({self.name}-{other.name})")
            try:
                new_stock.open = self.open - other.open
                new_stock.close = self.close - other.close
                new_stock.high = self.high - other.high
                new_stock.low = self.low - other.low
            finally:
                return new_stock.dropna()
        elif isinstance(other, TimeSeries):
            new_stock = Stock(self.data - other.data, name=f"({self.name}-Series)")
            try:
                new_stock.open = self.open - other.data
                new_stock.close = self.close - other.data
                new_stock.high = self.high - other.data
                new_stock.low = self.low - other.data
            finally:
                return new_stock.dropna()
        elif isinstance(other, numbers.Number):
            new_stock = Stock(self.data - other.data, name=f"{self.name}-{str(other)}")
            try:
                new_stock.high = self.high - other
                new_stock.low = self.low - other
                new_stock.open = self.open - other
                new_stock.close = self.close - other
            finally:
                return new_stock.dropna()

        else:
            raise Exception("Subtraction operation only supported for TimeSeries, Stock, int, and float.")

    def __neg__(self):
        new_stock = Stock(self.data, name=f"-({self.name})")
        new_stock.open = -(self.open)
        new_stock.close = -(self.close)
        new_stock.high = -(self.high)
        new_stock.low = -(self.low)
        return new_stock.dropna()

    def __mul__(self, other):
        if isinstance(other, Stock):
            new_stock = Stock(self.data * other.data,name = f"({self.name}*{other.name})")
            try:
                new_stock.open = self.open * other.open
                new_stock.close = self.close * other.close
                new_stock.high = self.high * other.high
                new_stock.low = self.low * other.low
            finally:
                return new_stock.dropna()
        elif isinstance(other, TimeSeries):
            new_stock = Stock(self.data * other.data,name = f"({self.name}*Series)")
            try:
                new_stock.open = self.open * other.data
                new_stock.close = self.close * other.data
                new_stock.high = self.high * other.data
                new_stock.low = self.low * other.data
            finally:
                return new_stock.dropna()
        elif isinstance(other,numbers.Number):
            new_stock = Stock(self.data * other.data, name = f"{self.name}*{str(other)}")
            try:
                new_stock.high = self.high * other
                new_stock.low = self.low * other
                new_stock.open = self.open * other
                new_stock.close = self.close * other
            finally:
                return new_stock.dropna()

        else:
            raise Exception("Multiplication for Stock only supported for TimeSeries, Stock, int, and float.")

    def __truediv__(self, other):
        if isinstance(other, Stock):
            new_stock = Stock(self.data / other.data,name = f"({self.name}/{other.name})")
            try:
                new_stock.open = self.open / other.open
                new_stock.close = self.close / other.close
                new_stock.high = self.high / other.high
                new_stock.low = self.low / other.low
            finally:
                return new_stock.dropna()
        elif isinstance(other, TimeSeries):
            new_stock = Stock(self.data / other.data,name = f"({self.name}/Series)")
            try:
                new_stock.open = self.open / other.data
                new_stock.close = self.close / other.data
                new_stock.high = self.high / other.data
                new_stock.low = self.low / other.data
            finally:
                return new_stock.dropna()
        elif isinstance(other,numbers.Number):
            new_stock = Stock(self.data / other.data, name = f"{self.name}/{str(other)}")
            try:
                new_stock.high = self.high / other
                new_stock.low = self.low / other
                new_stock.open = self.open / other
                new_stock.close = self.close / other
            finally:
                return new_stock.dropna()

        else:
            raise Exception("Division for Stock only supported for TimeSeries, Stock, int, and float.")

    def __pow__(self, other):
        if isinstance(other, Stock):
            new_stock = Stock(self.data ** other.data,name = f"({self.name}^{other.name})")
            try:
                new_stock.open = self.open ** other.open
                new_stock.close = self.close ** other.close
                new_stock.high = self.high ** other.high
                new_stock.low = self.low ** other.low
            finally:
                return new_stock.dropna()
        elif isinstance(other, TimeSeries):
            new_stock = Stock(self.data ** other.data,name = f"({self.name}^Series)")
            try:
                new_stock.open = self.open ** other.data
                new_stock.close = self.close ** other.data
                new_stock.high = self.high ** other.data
                new_stock.low = self.low ** other.data
            finally:
                return new_stock.dropna()
        elif isinstance(other,numbers.Number):
            new_stock = Stock(self.data ** other.data, name = f"{self.name}^{str(other)}")
            try:
                new_stock.high = self.high ** other
                new_stock.low = self.low ** other
                new_stock.open = self.open ** other
                new_stock.close = self.close ** other
            finally:
                return new_stock.dropna()

        else:
            raise Exception("Division for Stock only supported for TimeSeries, Stock, int, and float.")

    def getIndicators(self):
        return self.indicators

    def is_valid_interval(self, period):
        valid_period = ['1m', '2m', '5m', '15m', '30m', '60m', '90m','1h','1d','5d','1mo','3mo','6mo','1y','2y','5y','10y','ytd','max']
        if period in valid_period:
            return period
        else:
            raise("Interval must be one of '1m','2m','5m','15m','30m','60m','90m','1h','1d','5d','1mo','3mo','6mo','1y','2y','5y','10y','ytd','max'!")
            return None

    def update_data(self, ticker, period = None):
        self.data = yf.download(ticker, period)
        self.indicators = dict()

    def len(self):
        return super().__len__()

    def to_csv(self):
        return self.data.to_csv(self.name+'.csv')

    def earnings(self):  # , earnings_date: datetime):
        earning_data = si.get_earnings_history(self.ticker)
        df_eps = pd.DataFrame.from_dict(earning_data)
        eps_data = pd.DataFrame(df_eps["epsactual"])
        eps_data.index = qsc.round_dates(df_eps["startdatetime"])
        self.indicators["earning"] = eps_data
        return self.indicators["earning"]

    def eps_expected(self):
        earning_data = si.get_earnings_history(self.ticker)
        # print(qsc.round_dates(list(earning_data.index())))
        # earning_data.index = pd.Series(earning_data.index).dt.round('D').values
        df_eps = pd.DataFrame.from_dict(earning_data)
        # df_eps.index = pd.Series(df_eps.index).dt.round('D').values
        eps_data = df_eps["epsestimate"]
        eps_index = qsc.round_dates(df_eps["startdatetime"])
        eps_data.index = eps_index
        self.indicators["epsestimate"] = eps_data
        return self.indicators["epsestimate"]

    def eps(self):
        return self.earnings()

    def eps_suprise(self):
        earning_data = si.get_earnings_history(self.ticker)
        df_eps = pd.DataFrame.from_dict(earning_data)
        eps_data = df_eps["epssurprisepct"]
        eps_data.index = qsc.round_dates(df_eps["startdatetime"])
        self.indicators["epssurprisepct"] = eps_data
        return self.indicators["epssurprisepct"]

    def balance_sheet(self, yearly = True):
        self.indicators["balance_sheet"] = (si.get_balance_sheet(self.ticker, yearly) // 100).T
        return self.indicators["balance_sheet"]

    def cash_flow(self, yearly = True):
        self.indicators["cash_flow"] = (si.get_cash_flow(self.ticker, yearly) // 100).T
        return self.indicators["cash_flow"]

    def income_statement(self, yearly = True):
        self.indicators["income_statement"] = si.get_income_statement(self.ticker, yearly) // 100
        return self.indicators["income_statement"]

    def next_earnings_date(self):
        return si.get_next_earnings_date(self.ticker)

    def dividends(self): # can have specified date for onwards
        return si.get_dividends(self.ticker)


    def plot(self,backend=None, style='candle', figsize=(8,6),ylabel='Price'):
        if backend is None:
            backend = config.config['plot_backend']
        if backend == "matplotlib":
            fig,ax = super().get_fig(figsize = figsize, title=self.name, xlabel='Date',ylabel=ylabel)
            ax.plot(self.data)
            fig.show()
        elif backend == "plotly":
            if style == "candle":
                fig = go.Figure(data=[go.Candlestick(x=self.dates,
                        open=self.open,
                        high=self.high,
                        low=self.low,
                        close=self.close)])
                fig.update_layout(title_text = self.name,xaxis_title = 'Date', yaxis_title=ylabel)
            elif style == "line":
                fig = px.line(self.data)
            else:
                raise("type can only be 'candle' or 'line'")
            fig.show()
        else:
            raise("Backend must be either 'plotly' or 'matplotlib!'")

    # def sort_values(self, column = "Open"):
        #df = pd.DataFrame()
        #self.data

    # ""
    # def pairwise_sort(self):

    # def sort_index(self):
    # """
    # Reminder: Sort all self.data(default to 'open') self.open,self.close,self.high,self.low
    # """

    def diff(self,shift=1,inplace=False):
        if inplace:
            self.data = self.data.diff(shift)
            if None not in (self.high,self.low,self.open,self.close):
                self.high = self.high.diff(shift)
                self.low = self.low.diff(shift)
                self.open = self.open.diff(shift)
                self.close = self.close.diff(shift)
                self.diff_count += shift
                self.name = f"{self.ticker}.diff({str(self.diff_count)})"
            return self
        else:
            new_data = self.data.diff(shift)
            new_stock = Stock(new_data)
            new_stock.high = self.high.diff(shift)
            new_stock.low = self.low.diff(shift)
            new_stock.open = self.open.diff(shift)
            new_stock.close = self.close.diff(shift)
            new_stock.diff_count = self.diff_count + shift
            new_stock.name = f"{self.ticker}.diff({str(new_stock.diff_count)})"
            new_stock.ticker = self.ticker
            return new_stock.dropna()

    def autocov(self,lag=1):
        return super().autocov(lag)

    def autocorr(self, lag=1, method='pearson'):
        return super().autocorr(lag)

    def autocov_plot(self, figsize = (8,6), legend=False, title='', backend=None):
        return super().autocov_plot(figsize=figsize, legend=legend,
                                    title=f"Auto Covariance plot for {self.name}",backend=backend)
    def autocorr_plot(self, figsize = (8,6), legend=False, title='', backend=None,method='pearson'):
        return super().autocorr_plot(figsize=figsize, legend=legend,
                                    title=f"Auto Correlation plot for {self.name}",backend=backend,method=method)
    def var(self):
        return super().variance()

    def dropna(self):
        self.data = self.data.dropna()
        try:
            self.open = self.open.dropna()
            self.close = self.close.dropna()
            self.high = self.high.dropna()
            self.low = self.low.dropna()
        finally:
            return self

    def arima(self, p, d, q, plotResiduals=True, getSummary=True):
        super().arima(p, d, q, plotResiduals=True, getSummary=True)



# Allows user to call data.random.func instead of data.__random.func
# We need this since random is a default package in python and there's
# no way to work around that other than changing the source code of import

# add . before rand.py to let init know we are import a .py file instead of a module
from .data import *
#from data import __random


import warnings

import dateutil.parser
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import plotly.express as px
import plotly.graph_objects as go
import yfinance as yf
from pandas import DataFrame
from statsmodels.tsa.arima.model import ARIMA
from statsmodels.tsa.stattools import acf
from statsmodels.tsa.stattools import pacf

from quantsc import config


class TimeSeries:
    def __init__(self, data=None, start=None, end=None, interval='1d', delim=',', type='Open', nan=None):
        """

        :param data: numpy nd-array, pandas series, .csv .txt,ticker
        :param start:
        :param end:
        :param interval:
        :param delim:
        :param nan:
        """
        self.indicators = dict()
        self.data = None
        if isinstance(data, str):
            if not ('.csv' in data or '.txt' in data):
                self.data = np.round(yf.download(data, start=start, end=end, interval=interval)[type], 1)
        else:
            self.load_data(data, delim=delim)

    def __str__(self):
        if self.data is None:
            return ""
        else:
            return self.data.to_string()

    def __dict__(self):
        pass

    def load_data(self, data, delim=',', indicator=False):
        """Loads the input data into the timeseries object

        :param data: dict, array-like, pd.Series
        :return:
        """
        if isinstance(data, list):
            try:
                data = np.array(data)
            except:
                raise "Can't convert list to string!"

        # If the data is an array, default to using array index as time series index
        elif type(data).__module__ == np.__name__:
            if data.shape[1] == 1:
                self.data = pd.Series(data=data, index=list(range(len(data))) + 1)
            elif data.shape[1] == 2:
                self.data = pd.Series(data=data[:, 1], index=data[:, 0])
            return

        elif isinstance(data, dict):
            pass

        elif isinstance(data, pd.Series):
            self.data = data
            return

        # If the index of data is string, change the format to datetime
        elif isinstance(data, pd.DataFrame):
            series_data = data.iloc[:, 0]
            self.data = pd.Series(data=series_data, index=data.index)
            for i in range(1, data.shape[1]):
                indicator_name = data.columns[i]
                self.indicators[indicator_name] = data.iloc[:, i]
            return

        elif isinstance(data, str):
            # Check if data is a filename
            if ".csv" in data:
                df = pd.read_csv(data, sep=delim, header=None, index_col=0)
                df.columns.name = None
                self.load_data(df)
            elif ".txt" in data:
                pass
            return

            # Else, data is a string representing the time series

        raise "Only numpy arrays, pandas Series, and csv files are supported!"

        # Sort the time series data by index
        # If index is None, raise exception

    @staticmethod
    def check_integrity(self, series=None):
        """Checks if the given data is a valid time series, raise exception if data is not a valid time series

        :param series:
        :return:
        """
        data = series.data

        if not isinstance(data, pd.core.series.Series):
            raise "Time series data must be pd.Series!"

        ## Checks if all the indicies are in chronological order
        if not all(data.index[i] <= data.index[i + 1] for i in range(len(data.index) - 1)):
            raise "Time series are not in chronological order!"

        if not isinstance(self.data.index, pd.DatetimeIndex):
            raise "Index of the series must be datetime!"

        print("TimeSeries Successfully Validated")

    def plot(self, figsize=(6, 8), ylabel=None, title=None, plot_indicator=False, style=None):
        """Plots the TimeSeries

        :param figsize: tuple
        :param ylabel: str
        :param title: str
        :param plot_indicator:
        :param style:
        :return:
        """
        plt.figure(figsize=figsize)
        n = len(self.indicators) + 1
        if not plot_indicator:
            px.plot(self.data)
            plt.show()
        else:
            if style == 'together':
                for i in range(n):
                    plt.plot(self.indicators)
            else:
                col = n // 5
                row = 5
                fig, axs = plt.subplots(row, col)
                for i in range(n):
                    axs[i].plot(n)
                    axs.title(self.indicators[i].name)
                    axs.xlabel(self.indicators[i].name)
                    plt.subplot(1, 2, 1)

    def to_string(self, indicators=False):
        """Converts the TimeSeries to a string, includes indicators if specified

        :param indicators: boolean, str, or list
        :return: str
        """
        rslt_string = ""
        if self.data is not None:
            rslt_string += self.data.to_string()

        if isinstance(indicators, bool):
            if indicators:
                for name, indicator in self.indicators.items():
                    rslt_string += (str(self.indicators[name]) + "\n")
            else:
                return self.__str__()
        elif isinstance(indicators, str):
            return self.indicators[indicators] + "\n"
        else:
            for indicator in indicators:
                if indicator in self.indicators.keys():
                    rslt_string += str(self.indicators[indicator]) + "\n"

        return rslt_string

    def take_partial_autocorrelation(self, d):
        df = pd.DataFrame(self.data.values)
        for i in range(d):
            df = df.diff()
        plt.plot(pacf(df.dropna()))
        plt.show()

    def arima(self, p, d, q, plotResiduals=True, getSummary=True, ):
        df = pd.DataFrame(self.data.values)
        model = ARIMA(df, order=(p, d, q))
        model_fit = model.fit()
        if getSummary:
            print(model_fit.summary())
        print(type(model_fit.resid))
        residuals = DataFrame(model_fit.resid)
        if plotResiduals:
            fig, ax = plt.subplots(1, 2)
            residuals.plot(title="Residuals", ax=ax[0])
            residuals.plot(kind='kde', title='Density', ax=ax[1])
            plt.show()
        return model

    def __add__(self, other):
        if isinstance(other, TimeSeries):
            newData = (self.data + other.data).dropna()
            retVal = TimeSeries(newData)
            return retVal
        else:
            raise Exception("Second object in addition is not an instance of TimeSeries.")

    def __sub__(self, other):
        if isinstance(other, TimeSeries):
            newData = (self.data - other.data).dropna()
            retVal = TimeSeries(newData)
            return retVal
        else:
            raise Exception("Second object in subtraction is not an instance of TimeSeries.")

    def __mul__(self, other):
        if isinstance(other, TimeSeries):
            newData = (self.data * other.data).dropna()
            retVal = TimeSeries(newData)
            return retVal
        else:
            raise Exception("Second object in multiplication is not an instance of TimeSeries.")

            # for name,indicator_data in self.indicators.items():

    # def exp(self):

    def diff(self, shift=1, inplace=False):
        """

        :param shift:
        :param inplace:
        :return:
        """
        if inplace:
            self.data = self.data.diff(shift)
            return self
        else:
            return TimeSeries(self.data.diff(shift))

    def sort_values(self, ascending=True, inplace=False):
        if inplace:
            self.data = self.data.sort_values(ascending=ascending)
            return self
        else:
            sorted = self.data.sort_values(ascending=ascending)
            return TimeSeries(sorted)

    def sort_index(self, ascending=True, inplace=False):
        if inplace:
            self.data = self.data.sort_index(ascending=ascending)
        else:
            sorted = self.data.sort_index(ascending=ascending)
            return TimeSeries(sorted)

    def __len__(self):
        return self.data.size

    def pairwise_sort(self, other, ascending=True, inplace=False, plot=True):
        if not isinstance(other, TimeSeries):
            raise ("Pairwise sort only operates on two TimeSeries!")
        if other.data.size != self.data.size:
            raise ("Length of two TimeSeries has to be the same!")
        new_index = [self_index for self_data, self_index, other_index, self_index
                     in sorted(zip(other.data, self.data, other.data.index, self.data.index),reverse=not ascending)]
        new_data = self.data[new_index]
        # new_data =[self.data[self_index] for other_index,self_index
        #              in sorted(zip(other.data.index,self.data.index))]
        new_series = pd.Series(data=new_data, index=new_index)
        if inplace:
            self.data = new_series
            return self
        else:
            return TimeSeries(new_series)

    def autocorr(self, lag=1, method='pearson'):
        """

        :param lag:
        :param method: 'pearson','spearman','kendall'
        :return:
        """
        # return self.data.corr(self.data.shift(lag),method=method)
        return acf(self.data)[lag]

    def mean(self):
        return self.data.mean()

    def variance(self):
        return self.data.var()

    def autocov(self, lag=1):
        return self.data.cov(self.data.shift(lag))

    def autocov_plot(self, figsize, legend=False, title='', ylabel='Autocovariance', backend=None):
        if title == '':
            title = f"Autocovariance plot"
        if backend is None:
            backend = config.config['plot_backend']
        x_val = [i for i in range(len(self.data)) ]
        y_val = [self.autocov(x) for x in x_val]
        if backend == 'matplotlib':
            fig, ax = self.get_fig(figsize, xlabel='Lag', ylabel=ylabel, title=title)
            ax.plot(y_val, label="autocov")
            if legend:
                ax.legend()
            fig.show()
            return fig, ax

        elif backend == 'plotly':
            fig = go.Figure(data=go.Scatter(x=x_val, y=y_val))
            fig.show()
            return fig

    def autocorr_plot(self, figsize, legend=False, title='', ylabel='Autocorrelation', backend=None,method='pearson'):
        if backend is None:
            backend = config.config['plot_backend']
        y_val = acf(self.data)
        if backend == 'matplotlib':
            fig, ax = self.get_fig(figsize, xlabel='Lag', ylabel=ylabel, title=title)
            z95 = 1.959963984540054
            z99 = 2.5758293035489004
            n = len(self)
            ax.axhline(y=z99 / np.sqrt(n), linestyle="--", color="grey")
            ax.axhline(y=z95 / np.sqrt(n), color="grey")
            ax.axhline(y=0.0, color="black")
            ax.axhline(y=-z95 / np.sqrt(n), color="grey")
            ax.axhline(y=-z99 / np.sqrt(n), linestyle="--", color="grey")
            ax.plot(y_val, label="autocov")
            if legend:
                ax.legend()
            fig.show()
            return fig, ax

        elif backend == 'plotly':
            fig = go.Figure(data=go.Scatter(x=[x for x in range(len(y_val))], y=y_val))
            fig.show()
            return fig

    def cumsum(self,inplace=False):
        if inplace:
            self.data = self.data.cumsum()
            return self
        return self.data.cumsum()

    def covariance_matrix(self, other):
        pass

    def get_fig(self, figsize=(8, 6), xlabel='', ylabel='',title=None):
        """ Get a matplotlib.pyplot figure with the specified attributes

        :param figsize:
        :param title:
        :param xlabel:
        :param ylabel:
        :return:
        """
        if config.config['plot_backend'] == 'plotly':
            warnings.warn("get_fig only supports matplotlib figures!")
        fig = plt.figure(figsize=figsize)
        ax = fig.add_subplot(111)
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)
        ax.set_title(title)
        return fig, ax

    def between(self,start,end,inclusive=True):
        """ Gets a Stock object with time specified in the range of start to end

        :param start:
        :param end:
        :return: Stock Stock whose data is in between the range start and end
        """
        start = dateutil.parser(start)
        end = dateutil.parser(end)
        new_data = self.data.between(start,end,inclusive=inclusive)
        return TimeSeries(new_data)



        # covar = self.data.cov(self, i)

    # """
    # def plot_data(self,indicators=False):
    #     if indicators:
    #         total_indicators = len(indicators)
    #         fig,axs = plt.subplots(total_indicators // 2, 2)
    #         for idx,key,value in enumerate(indicators):
    #             axs[i]
    # """
    # def plot(self,indicators=False):
    #     if indicators:
    #         total_indicators = len(indicators)
    #         fig,axs = plt.subplots(total_indicators // 2, 2)
    #         for idx,key,value in enumerate(indicators):
    #             axs[i]



from setuptools import setup

setup(
    name='quantsc',
    version='1.0.1.3',
    packages=['quantsc', 'quantsc.core', 'quantsc.data', 'quantsc.data.__random'],
    url='https://quantsc.org/',
    license='MIT',
    author='Yuanhao Lu, Jonathan Qin, Aditya Prasad ',
    author_email='terryl@usc.edu',
    description='A Quantitative Finance Library',
    install_requires=[
       'setuptools',
       'pandas',
        'numpy',
        'yfinance',
        'statsmodels==0.13.2',
        'scipy',
        'matplotlib',
        'plotly',
        'python-dateutil',
        'yahoo_fin'
    ]
)
